//
//  @file KXR-L6_Walk_Sample.ino
//  @brief KXR-L6_Walk_Sample
//  @author Kondo Kagaku Co.,Ltd.
//  @date 2020/3/31
//
//  KXR-L6をArduino MEGAからサーボに直接角度指定して歩行
//  直線補間でサーボの挙動を制御
//
//  MEGAのUART1、UART2で通信


#include <IcsHardSerialClass.h>

const byte EN_PIN1 = 34;
const byte EN_PIN2 = 3;
const long BAUDRATE = 115200;
const int TIMEOUT = 1000;    //通信できてないか確認用にわざと遅めに設定

IcsHardSerialClass krs1(&Serial1,EN_PIN1,BAUDRATE,TIMEOUT);  //インスタンス＋ENピン(34番ピン)およびUART1の指定　左半身
IcsHardSerialClass krs2(&Serial2,EN_PIN2,BAUDRATE,TIMEOUT);  //インスタンス＋ENピン(3番ピン)およびUART2の指定　右半身


//脚を上げた時のポジション（MAX：600）
int UP_pos2 = 600;            //ID2, 5, 8
int UP_pos3 = UP_pos2 + 2666; //ID3, 6, 9

//脚を下げた時のポジション（MIN：-2800）
int DW_pos2 = -300;           //ID2, 5, 8
int DW_pos3 = DW_pos2 + 2666; //ID3, 6, 9

//pos2に対して2600を加算するとpos3がほぼ床に向きます。
//UP_pos2は数値が大きいほど脚を振り上げますが、600が限界値です。
//DW_pos2は数値が小さいほど脚を振り下ろしますが、-2600でまっすぐ伸ばした状態になります。
//上記の二つのパラメータを調整することで、地面からボディまでの距離と、
//脚の動作範囲を指定することができます。


void setup() {

  krs1.begin();  //サーボモータの通信初期設定
  krs2.begin();  //サーボモータの通信初期設定

  delay(1000);

  //ストレッチ設定
  for(int i = 1; i <= 9; i++){
    krs1.setStrc(i, 50);
    krs2.setStrc(i, 50);
  }
}


//////////////////////////////////////////////////////
//  補間制御
//////////////////////////////////////////////////////

void control_pos(
    int dpos0,  //左半身-ID1
    int dpos1,  //左半身-ID2
    int dpos2,  //左半身-ID3
    int dpos3,  //左半身-ID4
    int dpos4,  //左半身-ID5
    int dpos5,  //左半身-ID6
    int dpos6,  //左半身-ID7
    int dpos7,  //左半身-ID8
    int dpos8,  //左半身-ID9
    int dpos9,  //右半身-ID1
    int dpos10, //右半身-ID2
    int dpos11, //右半身-ID3
    int dpos12, //右半身-ID4
    int dpos13, //右半身-ID5
    int dpos14, //右半身-ID6
    int dpos15, //右半身-ID7
    int dpos16, //右半身-ID8
    int dpos17, //右半身-ID9
    int d_time, //補間間隔
    int hokan)  //補間回数
{

  int i = 0;    // カウント1
  int c = 0;    // カウント2

  int pos[18];  // ポジション
  int tpos[18]; // 目標値
  int hpos[18]; // ホームポジション
  int dpos[18]; // 指定値

  // ホームポジション
  // 二足歩行ロボットの直立した状態などホームポジションが
  // サーボのニュートラルでない場合は、ここにポジションを入れておきます。
  hpos[0] = 0;  //左半身-ID1
  hpos[1] = 0;  //左半身-ID2
  hpos[2] = 0;  //左半身-ID3
  hpos[3] = 0;  //左半身-ID4
  hpos[4] = 0;  //左半身-ID5
  hpos[5] = 0;  //左半身-ID6
  hpos[6] = 0;  //左半身-ID7
  hpos[7] = 0;  //左半身-ID8
  hpos[8] = 0;  //左半身-ID9
  hpos[9] = 0;  //右半身-ID1
  hpos[10] = 0; //右半身-ID2
  hpos[11] = 0; //右半身-ID3
  hpos[12] = 0; //右半身-ID4
  hpos[13] = 0; //右半身-ID5
  hpos[14] = 0; //右半身-ID6
  hpos[15] = 0; //右半身-ID7
  hpos[16] = 0; //右半身-ID8
  hpos[17] = 0; //右半身-ID9

  
  // 指定値を配列に代入
  dpos[0] = dpos0;
  dpos[1] = dpos1;
  dpos[2] = dpos2;
  dpos[3] = dpos3;
  dpos[4] = dpos4;
  dpos[5] = dpos5;
  dpos[6] = dpos6;
  dpos[7] = dpos7;
  dpos[8] = dpos8;
  dpos[9] = dpos9;
  dpos[10] = dpos10;
  dpos[11] = dpos11;
  dpos[12] = dpos12;
  dpos[13] = dpos13;
  dpos[14] = dpos14;
  dpos[15] = dpos15;
  dpos[16] = dpos16;
  dpos[17] = dpos17;


  for(i = 0; i < 9; i++)
  {
    // tpos[]に目標値を代入
    tpos[i] = 7500 + hpos[i] + dpos[i];
    tpos[i+9] = 7500 + hpos[i+9] + dpos[i+9];

    // サーボの現在値を取得(ICS3.6)
    pos[i] = krs1.getPos(i+1);
    pos[i+9] = krs2.getPos(i+1);

/*
    // サーボの現在値を取得(ICS3.5)
    // ICS3.5は現在値を取得するコマンドがありませんので
    // setFreeコマンドの戻り値で角度を読み込みます。
    pos[i] = krs1.setFree(i+1);
    pos[i+9] = krs2.setFree(i+1);
*/

    // 現在値から目標値へ動作するため
    // 目標値と現在値の距離を求める
    tpos[i] -= pos[i];
    tpos[i+9] -= pos[i+9];
    
    // 目標値までの距離を補間回数で割り
    // 1回あたりのポジションを求める
    tpos[i] /= hokan;
    tpos[i+9] /= hokan;
  }


  // 補間動作で全身のサーボモータを動かす
  for(i = 0; i < hokan; i++)
  {
    for(c = 0; c < 9; c++)
    {
      //現在値に補間回数で割った距離を順次加算し目標値に移動する
      pos[c] = pos[c] + tpos[c];
      krs1.setPos(c+1,pos[c]);    // 左半身の各サーボにポジションデータを送信する

      pos[c+9] = pos[c+9] + tpos[c+9];
      krs2.setPos(c+1,pos[c+9]);  // 右半身の各サーボにポジションデータを送信する
    }

    // サーボを滑らかに動かすように指定した間隔を空る
    delay(d_time);
  }
}


//////////////////////////////////////////////////////
//  ホームポジション
//////////////////////////////////////////////////////

void Home_Pos(int d_time, int hokan){
  
  int F_MID = 1200; //前脚-中心位置
  int M_MID = 0;    //中脚-中心位置
  int B_MID = 1200; //後脚-中心位置

  control_pos(F_MID * (-1), DW_pos2, DW_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_MID,        DW_pos2, DW_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_MID,        DW_pos2, DW_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右
}


//////////////////////////////////////////////////////
//  前進モーション
//////////////////////////////////////////////////////

void Walk_Forward(int c, int d_time, int hokan){

  int i;

  int F_BFR = 2600; //前脚-前に出す
  int F_MID = 1200; //前脚-中央位置
  int F_BAK = 2200; //前脚-後ろに下げる

  int M_BFR = 400;  //中脚-前に出す
  int M_MID = 0;    //中脚-中央位置
  int M_BAK = 0;    //中脚-後ろに下げる

  int B_BFR = 1800; //後脚-前に出す
  int B_MID = 1200; //後脚-中央位置
  int B_BAK = 2200; //後脚-後ろに下げる


  // 歩行ポーズに移行
  // control_pos(左半身:ID1, ID2, ID3, ID4, ID5, ID6, ID7, ID8, ID9
  //             右半身:ID1, ID2, ID3, ID4, ID5, ID6, ID7, ID8, ID9, 補間間隔, 補間回数);

  control_pos(F_MID * (-1), UP_pos2, UP_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_MID,        UP_pos2, UP_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_MID,        UP_pos2, UP_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右

  control_pos(F_BAK * (-1), UP_pos2, UP_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_BAK,        UP_pos2, UP_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_BAK,        UP_pos2, UP_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右

  control_pos(F_BAK * (-1), DW_pos2, DW_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_BAK,        DW_pos2, DW_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_BAK,        DW_pos2, DW_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右

  
  // 指定した回数前進
  for(i = 0; i < c; i++)
  {
    //右脚
    control_pos(F_BAK * (-1), DW_pos2, DW_pos3, M_BFR * (-1), UP_pos2, UP_pos3, B_BAK,        DW_pos2, DW_pos3,                   //左
                F_BFR,        UP_pos2, UP_pos3, M_BAK,        DW_pos2, DW_pos3, B_BFR * (-1), UP_pos2, UP_pos3, d_time, hokan);   //右

    control_pos(F_BFR * (-1), DW_pos2, DW_pos3, M_BAK * (-1), UP_pos2, UP_pos3, B_BFR,        DW_pos2, DW_pos3,                   //左
                F_BAK,        UP_pos2, UP_pos3, M_BFR,        DW_pos2, DW_pos3, B_BAK * (-1), UP_pos2, UP_pos3, d_time, hokan);   //右

    control_pos(F_BFR * (-1), DW_pos2, DW_pos3, M_BAK * (-1), DW_pos2, DW_pos3, B_BFR,        DW_pos2, DW_pos3,                   //左
                F_BAK,        DW_pos2, DW_pos3, M_BFR,        DW_pos2, DW_pos3, B_BAK * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右

    //左脚
    control_pos(F_BFR * (-1), UP_pos2, UP_pos3, M_BAK * (-1), DW_pos2, DW_pos3, B_BFR,        UP_pos2, UP_pos3,                   //左
                F_BAK,        DW_pos2, DW_pos3, M_BFR,        UP_pos2, UP_pos3, B_BAK * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右

    control_pos(F_BAK * (-1), UP_pos2, UP_pos3, M_BFR * (-1), DW_pos2, DW_pos3, B_BAK,        UP_pos2, UP_pos3,                   //左
                F_BFR,        DW_pos2, DW_pos3, M_BAK,        UP_pos2, UP_pos3, B_BFR * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右

    control_pos(F_BAK * (-1), DW_pos2, DW_pos3, M_BFR * (-1), DW_pos2, DW_pos3, B_BAK,        DW_pos2, DW_pos3,                   //左
                F_BFR,        DW_pos2, DW_pos3, M_BAK,        DW_pos2, DW_pos3, B_BFR * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右

  }

  // ホームポジションへ戻る動作
  control_pos(F_BAK * (-1), DW_pos2, DW_pos3, M_MID * (-1), UP_pos2, UP_pos3, B_BAK,        DW_pos2, DW_pos3,                 //左
              F_MID,        UP_pos2, UP_pos3, M_BAK,        DW_pos2, DW_pos3, B_MID * (-1), UP_pos2, UP_pos3, d_time, hokan); //右

  control_pos(F_BAK * (-1), DW_pos2, DW_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_BAK,        DW_pos2, DW_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_BAK,        DW_pos2, DW_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右

  control_pos(F_MID * (-1), UP_pos2, UP_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_MID,        UP_pos2, UP_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_MID,        UP_pos2, UP_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右

  // ホームポジション
  control_pos(F_MID * (-1), DW_pos2, DW_pos3, M_MID * (-1), DW_pos2, DW_pos3, B_MID,        DW_pos2, DW_pos3,                 //左
              F_MID,        DW_pos2, DW_pos3, M_MID,        DW_pos2, DW_pos3, B_MID * (-1), DW_pos2, DW_pos3, d_time, hokan); //右
 
  delay(500);
}


//////////////////////////////////////////////////////
//  旋回モーション
//////////////////////////////////////////////////////

//digの値で旋回する角度を指定できます。
//自然数で左旋回、マイナスで右旋回します。
//脚部の干渉の関係から、最大値は-1000～1000まで指定できます。

void Walk_Turn(int c, int dig, int d_time, int hokan){

  int i;

  int FB_MID = 1200;        //前後脚-中央位置
  int F_TRN = FB_MID + dig; //前脚-動作角
  int B_TRN = FB_MID - dig; //後脚-動作角

  int M_MID = 0;            //中脚-中央位置
  int M_TRN = M_MID + dig;  //中脚-動作角
  
  // 指定した回数旋回
  // control_pos(左半身:ID1, ID2, ID3, ID4, ID5, ID6, ID7, ID8, ID9
  //             右半身:ID1, ID2, ID3, ID4, ID5, ID6, ID7, ID8, ID9, 補間間隔, 補間回数);

  for(i = 0; i < c; i++)
  {
    control_pos(F_TRN * (-1), UP_pos2, UP_pos3, M_TRN,        DW_pos2, DW_pos3, B_TRN,        UP_pos2, UP_pos3,                   //左
                F_TRN,        DW_pos2, DW_pos3, M_TRN * (-1), UP_pos2, UP_pos3, B_TRN * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右

    control_pos(F_TRN * (-1), DW_pos2, DW_pos3, M_TRN,        DW_pos2, DW_pos3, B_TRN,        DW_pos2, DW_pos3,                   //左
                F_TRN,        DW_pos2, DW_pos3, M_TRN * (-1), DW_pos2, DW_pos3, B_TRN * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右

    control_pos(FB_MID * (-1), DW_pos2, DW_pos3, M_MID,        UP_pos2, UP_pos3, FB_MID,        DW_pos2, DW_pos3,                   //左
                FB_MID,        UP_pos2, UP_pos3, M_MID * (-1), DW_pos2, DW_pos3, FB_MID * (-1), UP_pos2, UP_pos3, d_time, hokan);   //右

    control_pos(FB_MID * (-1), DW_pos2, DW_pos3, M_MID,        DW_pos2, DW_pos3, FB_MID,        DW_pos2, DW_pos3,                   //左
                FB_MID,        DW_pos2, DW_pos3, M_MID * (-1), DW_pos2, DW_pos3, FB_MID * (-1), DW_pos2, DW_pos3, d_time, hokan);   //右
  }

  delay(500);
}


void loop() {

  //ゆっくりとホームポジションへ
  Home_Pos(30, 60);

  //前進する
  Walk_Forward(5, 3, 15);       //(歩行回数, 補間間隔, 補間回数)

  //姿勢を高くする（MIN：-2800）
  DW_pos2 = -2000;           //ID2, 5, 8
  DW_pos3 = DW_pos2 + 2666; //ID3, 6, 9

  //前進する
  Walk_Forward(5, 3, 40);       //(歩行回数, 補間間隔, 補間回数)

  //姿勢を低くする（MIN：-2800）
  DW_pos2 = -300;           //ID2, 5, 8
  DW_pos3 = DW_pos2 + 2666; //ID3, 6, 9

  //旋回する
  Walk_Turn(3, -1000, 3, 20);    //(歩行回数, 旋回角度, 補間間隔, 補間回数)

}
